'use strict';
var Benchmark = require('benchmark'),
    suite = new Benchmark.Suite(),
    Int64 = require('node-int64');

var MAX_UINT = Math.pow(1<<16, 2);
var MAX_SAFE_HIGH_BITS = Math.pow(2, 53 - 32);
function withCheckBeforeInt64(buffer) {
  var high = buffer.readUInt32BE(0);
  var low = buffer.readUInt32BE(4);
  if (Math.abs(high) < MAX_SAFE_HIGH_BITS)
    return high * MAX_UINT + (low >>> 0);

  var int64 = new Int64(buffer);
  var number = int64.toNumber(false);
  if (isFinite(number)) return number;
  return int64;
}

function withCheckBeforeInt64Long(buffer) {
  var high = buffer.readUInt32BE(0);
  var low = buffer.readUInt32BE(4);
  if (high < MAX_SAFE_HIGH_BITS && high > -MAX_SAFE_HIGH_BITS)
    return high * MAX_UINT + (low >>> 0);

  var int64 = new Int64(buffer);
  var number = int64.toNumber(false);
  if (isFinite(number)) return number;
  return int64;
}

suite
  .add('with check before int64 (ulong-abs)', function() {
    withCheckBeforeInt64(new Buffer([0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00]));
  })
  .add('with check before int64 (ulong)', function() {
    withCheckBeforeInt64Long(new Buffer([0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00]));
  })
  .on('cycle', function(event) { console.log(String(event.target)); })
  .on('error', function(err) { console.log(err); })
  .run();
